function [x,psi,rho]=psirho(obj,gate_spacing,xmin,xmax)
% PSIRHO determines the wave function and the probability density of a 
%   physical object which interacts with a step of potential energy. 
%   Computing is made over an interval on Ox axis between xmin and xmax.
%
%   [x,psi,rho]=psirho(obj,gate_spacing,xmin,xmax)
%       >> obj           = Step object
%       >> gate_spacing  = Gate spacing on axis Ox
%       >> xmin          = Lowest limit on axis Ox
%       >> xmax          = Highest limit on axis Ox
%       << x             = Variable of position on Ox axis
%       << psi           = Wave function
%       << rho           = Probability density
%
%   Olivier Pujol (2010)
%
if (isa(obj,'step')==0),error('Bad object type!');end
if (gate_spacing>=0.01),error('Gate spacing on Ox could be too large!');end
if ( (xmin>=obj.x)||(xmax<=obj.x) ),error('Step object out of domain!');end
%
mat=matrix(obj);
%
x=cell(1,2);
psi=cell(1,2);
rho=cell(1,2);

%   Input
x{1}=linspace(xmin,obj.x,1+(obj.x-xmin)/gate_spacing);
psi{1}=nan(size(x{1}));
psi{1}=(1/sqrt(obj.ke))*(exp(1i*obj.ke*(x{1}-obj.x)) ...
                        + (-mat(2,1)/mat(2,2))*exp(-1i*obj.ke*(x{1}-obj.x)) );
rho{1}=nan(size(x{1}));
rho{1}=psi{1}.*conj(psi{1});

%   Output
x{2}=linspace(obj.x,xmax,1+(xmax-obj.x)/gate_spacing);
psi{2}=nan(size(x{2}));
psi{2}=(1/sqrt(obj.ks))*(1/mat(2,2))*exp(1i*obj.ks*(x{2}-obj.x));
rho{2}=nan(size(x{2}));
rho{2}=psi{2}.*conj(psi{2});

% Normalization of the functions psi and rho by the area below rho(x) between
% xmin and xmax.
area =  0.5*sum((rho{1}(1:end-1)+rho{1}(2:end))*gate_spacing) + ...
        0.5*sum((rho{2}(1:end-1)+rho{2}(2:end))*gate_spacing)/2;
rho{1}=rho{1}/area;
rho{2}=rho{2}/area;
%
psi{1}=psi{1}/sqrt(area);
psi{2}=psi{2}/sqrt(area);

return;