function [sys,E]=EpLin(slope,m,varargin)
% EpLin determines the bound state energies of a physical object which interacts 
%   with a linear potential energy of slope a (Ep = a*x). 
%
%   [sys,E]=EpLin(slope,m,varargin) 
%					-- varargin states for various input arguments.
%   >> slope   	= Slope of the potential energy (J/m)
%   >> m        	= Mass of the physical object (with respect to electron mass)
%   >> varargin  	= Optional input arguments:
%        >> nx   	= Number of Ox-gates (default: 100)
%        >> nE   	= Number of energie values (default: 1000)
%        >> Emin 	= Lowest value of energy (default: min(Ep))
%        >> Emax 	= Highest value of energy (default: Ep(max(x)))
%   
% Olivier Pujol (2010)

%% Verifications

if (length(varargin)>4), error('Bad number of optional input arguments');end
if (nargin-length(varargin)~=2),error('Bad number of non optional input arguments');end
if (slope<=realmin),error('Slope is zero');end

%% Characteristic length. Conversion of the slope in eV/nm

% Constants
h=6.6260755E-34;    % Planck constant (J.s)
hbar=h/(2*pi);
eV=1.60217733E-19;  % Electronvolt (J) 
me=0.91093897E-30;  % Electron mass (kg)

Lc=(hbar^2/(2*m*me*abs(slope)))^(1/3);   % Characteristic length (in m)
Lc=1E9*Lc;                         		 % Conversion in nanometre

slope=1E-9*slope/eV;                	 % Slope J/m --> eV/nm

%% Optional input arguments

switch length(varargin)
    case 0
        nx=100;                     % Default value of Ox gate number
        nE=1000;                    % Default value of energy values number
    case 1
        nx=varargin{1};             % Default value of Ox gate number
        nE=1000;                    % Default value of energy values number
    case 2
        nx=varargin{1};             % Default value of Ox gate number
        nE=varargin{2};             % Default value of energy values number
    case 4
        nx=varargin{1};   			% Value of Ox gate number          
        nE=varargin{2};             % Value of energy values number
        E=linspace(varargin{3},varargin{4},nE);     % Set of energy values
    otherwise
        ('Bad number of optional input arguments');
end

%% Potential energy

% Construction
x=linspace(0,10*Lc,nx);     % Spatial domain 
Epc=slope*x;                % Linear potential energy (slope in eV/nm)

% Digitalization
[M,xt,Ep]=digitalization(x,Epc);

% If the slope is positive, a new step, M0, of 100*Ep(end) height 
% is added at xt=0. If the slope is negative, the step M0 is added at
% xt=max(x); step height is 100*abs(Ep(end))

if (slope>0)
    M=['M0' M];                             
    xt=[0;xt];
    Ep=[100*Ep(end);Ep];
end
if (slope<0)
   M=[M 'M0'];                             
   xt=[xt;max(x)];
   Ep=[Ep;100*abs(Ep(end))];
end 

% Energy values
if (length(varargin)~=4)
    if (slope>0),E=linspace(min(Ep),Ep(end),nE);end
    if (slope<0),E=linspace(min(Ep),0,nE);end
end    

if (slope>0 && max(E)>Ep(end)),error('Some E-values are out of bound states domain');end
if (slope<0 && max(E)>0),error('Some E-values are out of bound states domain');end

%% Bound states

[sys,E]=Epot(M,xt,Ep,m,E);          % System
sysconf(sys,E);                     % Bound states

return;