function [sys,E]=OscAnHarm(L,Ep0,C,m,varargin)
% OscAnHarm returns the bound state energies of a physical object confined in a 
%  cubic anharmonic well of potential energy
%  
%  [sys,E]=OscAnHarm(L,Ep0,C,m,varargin) 
%					-- varargin states for various input arguments
%   	>> L    	 = Well width (nm)
%   	>> Ep0  	 = Well depth (eV) (Ep0 < 0)
%   	>> C    	 = Constant 
%   	>> m    	 = Physical object mass (with respect to mass electron)
%   	>> varargin  = Optional input arguments
%           > nx	 = Gate number on Ox axis (default value: 100)
%           > nE	 = Number of energy values (default: 500)
%           > Emin 	 = Lowest energy value (default: min(Ep))
%           > Emax 	 = Highest energy value (default: 0)
%
% Olivier Pujol (2010)

%% Verifications

if (length(varargin)>4), error('Bad number of optional input arguments');end
if (nargin-length(varargin)~=4),error('Bad number of non optional input arguments');end

%% Verification about C value 

if (C==0),error('C=0! This is a harmonic oscillator');end

Clim=16*abs(Ep0)/(3*sqrt(3)*L^3);
if (abs(C)>Clim),error('C value is too large');end        

clear Clim

%% Well bounds determination

polynome=[-C 4*Ep0/L^2 0 -Ep0];     % Three real roots if abs(C)<Clim
racine=roots(polynome);             % Polynom roots

if (isreal(racine)==0)              % Are roots real?
    error('There are complex roots (C value is too large)');
end

% The spatial limits of the well are given by the two lowest roots. The largest one is not interesting herein.

        if (C>0)
            xA=racine(3);    % Lower limit
            xB=racine(2);    % Upper limit
        elseif (C<0)
            xA=racine(2);    % Lower limit
            xB=racine(3);    % Upper limit
        end

if (xA>xB)
    error('Incorrect spatial domain: lower limit is higher than the higher one');
end

clear polynome racine

%% Optional input arguments

switch length(varargin)
    case 0
        nx=100;                     % Default value of Ox gate number
        nE=500;                     % Default value of energy values number
    case 1
        nx=varargin{1};             % Default value of Ox gate number
        nE=500;                     % Default value of energy values number
    case 2
        nx=varargin{1};             % Default value of Ox gate number
        nE=varargin{2};             % Default value of energy values number
    case 4
        nx=varargin{1};             % Value of Ox gate number
        nE=varargin{2};             % Value of energy values number
        E=linspace(varargin{3},varargin{4},nE);     % Set of energy values
    otherwise
        ('Bad number of optional input arguments');
end

%% Potential energy

% Construction
x=linspace(xA,xB,nx);             % Spatial domain
clear xA xB
Epc=4*Ep0*(x/L).^2-C*x.^3;        % Potential energy (cubic anharmonic perturbation)

% Digitalization
[M,xt,Ep]=digitalization(x,Epc);    % Digitalization

% Energy values
if (length(varargin)~=4)
    E=linspace(min(Ep),Ep0,nE);       % Set of energy values
end    

if (max(E)>Ep0)
    error('Some E values are out of bound states domain');
end

%% Bound states

[sys,E]=Epot(M,xt,Ep,m,E);          % System
sysconf(sys,E);                     % Bound states

return;