function [En,psin,rhon]=Schrodinger(xmin,xmax,Ep,m,num_sol)
% Solve Schrdinger's equation for energy eigenvalues and eigenfunctions, 
%  i.e. energies of stationnary states.
%  It can be useful, especially for bound (o confined) states since, sometimes, 
%  the transfer-matrix method is too sensible to the numerical approximations.
%
%   [En,psin,rhon]=Schrodinger(xmin,xmax,Ep,m,num_sol)
%
%   	>> xmin 	= 	Lowest value of x on Ox axis (nm)
%   	>> xmax 	= 	Highest value of x on Ox axis (nm)
%   	>> Ep		=	Potential energy (eV)
%   	>> m 		=	Physical object mass (with respect to the electron mass)
%   	>> num_sol	=	Number of eigenvalues (energies) wanted
%		<< En		=	Bound state energies (eV)
%		<< psin		= 	Wave function which corresponds to to En
%		<< rhon		=  	Probability density which corresponds to En
%
% Olivier Pujol (2010)   
%
%% Constants and conversions

h=6.6260755E-34;    % Planck constant (J.s)
hbar=h/(2*pi);
hbar2=hbar^2;
me=0.91093897E-30;  % Electron mass (kg)
eV=1.60217733E-19;  % Electronvolt (J) 
nm=1E-9;            % Nanometer (m)

m=m*me;             % Mass conversion

n=length(Ep);       		% Number of sampling points
samp_step=(xmax-xmin)/n;  	% Sampling step (nm)

cte=(hbar2)/(2*m*eV*(nm*samp_step)^2);    % Constant which appears in Schrdinger's equation 

clear hbar hbar2 m me eV nm

%% Hamiltonian matrix

dia=nan(1,n-1);                     % Diagonal elements initialization  
nondia=nan(1,n-1);                  % Non diagonal elements initialization  

for i=2:n
    if min(Ep)<0
        dia(i-1)=2*cte+Ep(i)-min(Ep);           % Diagonal element
    else
        dia(i-1)=2*cte+Ep(i);           		% Diagonal element
    end
    nondia(i-1)=cte;                			% Non diagonal element
end

d1=-nondia(2:n-1);							
Hamilt=diag(d1,1);                  % Upper diagonal of the hamiltonian							
Hamilt=Hamilt+diag(d1,-1);          % Lower diagonal of the hamiltonian

d2=dia(1:n-1);
Hamilt=Hamilt+diag(d2,0);           % Diagonal of the hamiltonian

clear i cte dia nondia d1 d2 

%% Eigenvalue equation solving H\psi = E\psi

[psi,E]=eigs(Hamilt,num_sol,'SM');			% Use of matlab function eigs to find num_sol eigenfunctions and eigenvalues

clear Hamilt

% Eigenvalues
En=nan(1,num_sol);
for i=1:num_sol
    En(i)=E(i,i)+min(Ep);
end

clear E

% Eigenvectors
psin=[zeros(1,num_sol);psi;zeros(1,num_sol)];  % Wave function is zero at x = 0 and x = xmax

clear psi

%% Computing of the normalized probability density

rhon=abs(psin).^2;

% normalization
area=nan(1,size(rhon,2));
for in=1:size(rhon,2)
    area(in)=sum((rhon(1:end-1,in)+rhon(2:end,in))*samp_step)/2;    
    rhon(:,in)=rhon(:,in)/area(in);
    psin(:,in)=psin(:,in)/sqrt(area(in));
end

clear area in samp_step

%% Graphical representation

nt=input('Number of plot to make (1 <= nt <= 7) ');

x=linspace(xmin,xmax,n);

if (nt> num_sol),error('The number considered is greater than the number of eigenvalues');end
if (nt<=0),error('The number considered must be greater than or equal to 1');end
if (nt>=7),error('The number considered must be lower than or equal to 7');end

for nfig=1:nt
    figure(nfig)
    subplot(2,1,1)
    plot(x,Ep);
    if (min(Ep)<0),axis([xmin xmax min(Ep) 0]);end
    if (min(Ep)>=0),axis([xmin xmax min(Ep) Ep(end)]);end
    set(gca,'xminortick','on','yminortick','on');
    grid on;
    box on
    xlabel('{\itx} (nm)');
    title('{Potential energy (eV)}');
    %
    subplot(2,1,2)
    plot(x,rhon(1:end-1,nt-nfig+1),'r');
    set(gca,'xminortick','on','yminortick','on');
    grid on;
    box on
    axis tight
    xlabel('{\itx} (nm)');
    ylabel(['{|\it\psi_',num2str(nfig),' |^{2}}']);
end

clear ifig

if (nt==1),return;end

% Caption for next figure 

Leg=cell(1,nt);
for il=1:nt
    Leg{il}=['{|\it\psi_',num2str(il),' |^{2}}'];
end
clear il
%
figure
subplot(2,1,1)
plot(x,Ep);
if (min(Ep)<0),axis([xmin xmax min(Ep) 0]);end
if (min(Ep)>=0),axis([xmin xmax min(Ep) Ep(end)]);end
set(gca,'xminortick','on','yminortick','on');
grid on;
box on
xlabel('{\itx} (nm)');
title('{Potential energy (eV)}');
%
subplot(2,1,2)
C=['r','b','k','g','m','y','c'];
for it=1:nt     
    plot(x,rhon(1:end-1,nt-it+1),C(it));
    hold on
end
legend(Leg)
set(gca,'xminortick','on','yminortick','on');
grid on;
box on
axis tight
xlabel('{\itx} (nm)');
ylabel('{|\it\psi |^{2}}');

return