function k=WaveVect(E,m,Ep)
% WaveVect returns the complex wavevector k of a physical object using its 
%  energy, its mass, and the values of the potential energy. 
%
%   k=WaveVect(E,m,Ep)
%       >> E    = Physical object energy (eV)
%       >> m    = Physical object mass (with respect to electron mass)
%       >> Ep   = Potential energies Ep(i) (eV), (i=1,2,...) 
%       << k    = Wavevectors k(i) (nm^-1)
%                        
%   Olivier Pujol (2010)
%
%% Verification

if (m<=0),error('Mass must be positive!');end

%% Constants and conversions

% Fundamental constants
h=6.6260755E-34;    %   Planck constant (J.s)
hbar=h/(2*pi);
hbar2=hbar^2;
%
me=0.91093897E-30;  %   Electron mass (kg)
eV=1.60217733E-19;  %   Electronvolt (J) 
%
% Conversions
m=m*me;
E=E*eV;
nm=1E-9;

%% Wavevectors computing

k=nan(size(Ep));
if (length(m)>1)
    for n=1:length(Ep)
        Ep(n)=Ep(n)*eV;
        if (Ep(n)<E), k(n)=sqrt( (2*m(n)/hbar2).*(E-Ep(n)) );end
        if (Ep(n)>E), k(n)=1i*sqrt( (2*m(n)/hbar2).*(Ep(n)-E) );end
        if (Ep(n)==E),error('One of the values of E leads to the singular case k=0 !');end
    end
end
if (length(m)==1)
    for n=1:length(Ep)
        Ep(n)=Ep(n)*eV;
        if (Ep(n)<E), k(n)=sqrt( (2*m/hbar2)*(E-Ep(n)) );end
        if (Ep(n)>E), k(n)=1i*sqrt( (2*m/hbar2)*(Ep(n)-E) );end
        if (Ep(n)==E),error('One of the values of E leads to the singular case k=0!');end
    end
end
%
k=k*nm;
%
return;